/*============================================================================
 * rtc_timer.h
 *      Header file for RTC and timers.
 *==========================================================================*/

#ifndef _rtc_timer_h_
#define _rtc_timer_h_

/* OSCEN: Oscillator enable register */

#define OSCEN_ADOEN                     0x4             // FRC oscillator
#define OSCEN_SOSCEN                    0x8             // 32KHz crystal oscillator for RTC
#define OSCEN_LFOEN                     0x10            // 31kHz internal oscillator
#define OSCEN_MFOEN                     0x20            // Medium frequency internal osc.
#define OSCEN_HFOEN                     0x40            // High frequency internal osc.
#define OSCEN_EXTOEN                    0x80            // External oscillator

/* T1CON: Timer1 Control Register Defines. */

#define T1CON_PRESCALE_8                (3 << 4)
#define T1CON_PRESCALE_4                (2 << 4)
#define T1CON_PRESCALE_2                (1 << 4)
#define T1CON_PRESCALE_1                (0 << 4)

#define T1CON_READ_8BIT                 (0 << 1)
#define T1CON_READ_16BIT                (1 << 1)

#define T1CON_ASYNC                     (1 << 2)

#define T1CON_ON                        (1 << 0)


/* T1CLK: Timer1 Clock source defines. */

#define T1CLK_SOSC                      0x07

#define PIR4_TMR1IF   (1 << 0)

// Convert ticks on timer1 to/from ms.  Need to change this if you change
// the prescaler.
#define TIMER1_TO_MS(tmr1)    (((uint32_t)(tmr1)*1000)/32768)
#define MS_TO_TIMER1(ms)      ((uint32_t)(ms)*32768/1000)

typedef struct
{
    uint8_t hour, min, sec;
    uint8_t year, month, day;
} rtc_time_def;

extern rtc_time_def rtc_time;

void rt_init(void);
uint16_t rt_timer_read(void);
uint16_t rt_timer_to_ms(uint16_t t1, uint16_t t2);
void rt_delay(unsigned short ms);

void rtc_init(void);
void rtc_set_date(void);
void rtc_set_time(void);
void rtc_get(void);
void rtc_user_set_date(void);
void rtc_user_set_time(void);
void rtc_print_date_time(rtc_time_def *t, int format);
uint32_t rtc_date_to_timestamp(rtc_time_def *t);
void rtc_timestamp_to_date(uint32_t ts, rtc_time_def *t);
void rt_sleep(void);
void rt_sleep_off(void);


#endif
