/*============================================================================
 * tag_log.h
 *      Header file for RFID tag logging code.
 *==========================================================================*/

#ifndef _tag_log_h_
#define _tag_log_h_

#include <inttypes.h>
#include "global.h"

typedef uint16_t log_pointer_def;
typedef uint16_t log_index_def;

typedef struct
{
    uint32_t time_stamp;
    uint16_t antenna_mask;
    uint8_t rfid[5];
    //uint8_t reserved;  // I don't think it needs to be aligned.
    
} log_entry_def;

typedef struct
{
    uint8_t valid;
    uint8_t rfid[5];
    uint32_t start_time_stamp;
    uint32_t start_time_ms;
    uint16_t antenna_hits[MAX_ANTENNAS];

} tag_cache_entry_def;

extern tag_cache_entry_def tag_cache[MAX_CACHE_ENTRIES];

#define LOG_POINTER_ADDRESS(idx) ((uint32_t)EEPROM_LOG_POINTER_BASE + ((uint32_t)idx)*(uint32_t)sizeof(log_pointer_def))
#define LOG_POINTER_MAX          (EEPROM_LOG_POINTER_SIZE/sizeof(log_pointer_def))
#define LOG_ENTRY_ADDRESS(idx)   ((uint32_t)EEPROM_LOG_DATA_BASE + ((uint32_t)idx)*(uint32_t)sizeof(log_entry_def))
#define LOG_ENTRY_MAX            (EEPROM_LOG_DATA_SIZE/sizeof(log_entry_def))

#define TAG_LOG_CODE_POWER      0x01
#define TAG_LOG_CODE_SLEEP      0x02
#define TAG_LOG_CODE_WAKE       0x03

void tag_log_init(void);
int8_t tag_log_erase_log_pointers(void);
int8_t tag_log_get_pointer(log_index_def *found_idx, log_pointer_def *found_val);
int8_t tag_log_store(uint8_t ant_idx, uint8_t *rfid, uint32_t time_stamp);
int8_t tag_log_erase(void);
int8_t tag_log_dump(void);
int8_t tag_log_dump_formatted(void);
void tag_log_run(void);
void tag_log_write_code(uint8_t sub_code);

#endif
