/*============================================================================
 * test.c
 *      Code for testing various things on RFID reader.
 * Author: Jon W. Wallace 
 *==========================================================================*/

#include <xc.h>
#include <inttypes.h>

#include "global.h"

#include "uart.h"
#include "rtc_timer.h"
#include "timer_ms.h"
#include "i2c.h"
#include "eeprom.h"
#include "parse.h"
#include "rfid.h"
#include "adc.h"
#include "battery.h"
#include "command.h"
#include "test.h"
#include "config.h"
#include "switch.h"
#include "main.h"
#include "tag_log.h"

#define TEST_RUNTIME_MS         1000

// Borrow these directly from command.c to save memory space.
extern uart_char input_string[64];
extern uint8_t byte_buffer[16];
extern uint32_t word_buffer[16];

uart_char *input_string_test;

// Borrow function from tag_log.c for testing only.
int8_t tag_log_store_entry_eeprom(void);
extern log_entry_def log_entry;

void test_init()
{
    // Get the current time and use it to seed random number generator
    rtc_get();
    srand(rtc_date_to_timestamp(&rtc_time));
}

/*
void i2c_test()
{
    uint16_t byte_count, i;
    uint8_t address;
    uint8_t byte1;
    
    if (input_string_test[1] == 'r')
    {
        // i2c read
        if (parse_get_numbers((parse_char *)(input_string_test+2),
                              byte_buffer,
                              sizeof(byte_buffer),
                              &byte_count,
                              PARSE_INTEGER_SIZE_UINT8,
                              PARSE_RADIX_HEX)
                              < 0)
        {
            goto backout_error;
        }
        address = byte_buffer[0];
        byte_count = byte_buffer[1];

        if (byte_count > sizeof(byte_buffer))
        {
            goto backout_error;
        }
        
        if (i2c_start(address, I2C_START_READ, I2C_ACK) < 0) 
        {
            uart_write_string("No ACK on start.\n");
            goto backout_stop;
        }

        for (i=0; i<byte_count; i++)
        {
            if (i2c_read_byte(&byte1, (i != (byte_count-1))) < 0)
            {
                uart_write_string("No ACK on read.\r\n");
                goto backout_stop;
            }
            byte_buffer[i] = byte1;
        }
        
        i2c_stop();
        
        for (i=0; i<byte_count; i++)
        {
            uart_write_hex_byte(byte_buffer[i]);
            uart_write(' ');
        }
        uart_write_string("\r\n");
    }
    else if (input_string_test[1] == 'w')
    {
        // i2c write
        if ((parse_get_numbers((parse_char *)(input_string_test+2),
                              byte_buffer,
                              sizeof(byte_buffer),
                              &byte_count,
                              PARSE_INTEGER_SIZE_UINT8,
                              PARSE_RADIX_HEX)
                              < 0) || (byte_count < 2))
        {
            goto backout_error;
        }

        if (i2c_start(byte_buffer[0], I2C_START_WRITE, I2C_ACK) < 0)
        {
            uart_write_string("No ACK on write.\r\n");
            goto backout_stop;
        }

        for (i=0; i<(byte_count-1); i++)
        {
            if (i2c_write_byte(byte_buffer[i+1], 1) < 0)
            {
                uart_write_string("I2C error on data write.\r\n");
            }
        }

        i2c_stop();
    }
    else
    {
        uart_write_string("Usage:\r\n");
        uart_write_string("iw [address] [data] ... \r\n");
        uart_write_string("ir [address] \r\n");
        goto backout_base;
    }
    return;

 backout_stop:
    i2c_stop();
    return;
    
 backout_error:
    command_error();

 backout_base:
    return;

}
*/

void eeprom_test()
{
    uint16_t num_count;
    uint16_t i;
    
    if (input_string_test[1] == 'r')
    {
        // EEPROM read
        if ((parse_get_numbers((parse_char *)(input_string_test+2),
                               word_buffer,
                               sizeof(word_buffer),
                               &num_count,
                               PARSE_INTEGER_SIZE_UINT32,
                               PARSE_RADIX_HEX) < 0) ||
            (num_count != 2))
        {
            goto backout_error;
        }

        if (word_buffer[1] > sizeof(byte_buffer))
        {
            uart_write_string("Count too large.\r\n");
            goto backout_error;
        }

        if (eeprom_read_bytes(word_buffer[0], byte_buffer, (uint8_t) word_buffer[1]) < 0)
        {
            uart_write_string("EEPROM read failed.\r\n");
            goto backout_error;
        }
        
        for (i=0; i<word_buffer[1]; i++)
        {
            uart_write_hex_byte(byte_buffer[i]);
            uart_write(' ');
        }
        uart_write_string("\r\n");
    }
    else if (input_string_test[1] == 'w')
    {
        // i2c write
        if ((parse_get_numbers((parse_char *)(input_string_test+2),
                               word_buffer,
                               sizeof(word_buffer),
                               &num_count,
                               PARSE_INTEGER_SIZE_UINT32,
                               PARSE_RADIX_HEX) < 0) ||
            (num_count < 2))
        {
            goto backout_error;
        }

        for (i=0; i<num_count-1; i++)
        {
            byte_buffer[i] = (uint8_t)(word_buffer[i+1] & 0xff);
        }
        
        if (eeprom_write_bytes(word_buffer[0], byte_buffer, num_count-1) < 0)
        {
            uart_write_string("EEPROM write failed.\r\n");
            goto backout_error;
        }

    }
    else
    {
        uart_write_string("Usage:\r\n");
        uart_write_string("ew [address] [data] ... \r\n");
        uart_write_string("er [address] \r\n");
        goto backout_base;
    }
    return;

 backout_stop:
    i2c_stop();
    return;
    
 backout_error:
    command_error();

 backout_base:
    return;

}

/*
void rfid_test()
{
    uint32_t i;
    uint16_t word_count;
    uint8_t verbose;
    uint8_t ant_idx;

    uint8_t ant_start;
    uint8_t ant_end;
    uint16_t count;
    uint32_t poll_ms;
    uint32_t wait_ms;
    
    if ((parse_get_numbers((parse_char *)(input_string_test+2),
                           word_buffer,
                           sizeof(word_buffer),
                           &word_count,
                           PARSE_INTEGER_SIZE_UINT32,
                           PARSE_RADIX_DEC)
         < 0) || (word_count < 5))
    {
        uart_write_string("r [ant_start] [ant_end] [count] [poll_ms] [wait_ms] [verbose?]\n");
        return;
    }

    ant_start = (uint8_t)word_buffer[0];
    ant_end = (uint8_t)word_buffer[1];
    count = (uint8_t)word_buffer[2];
    poll_ms = word_buffer[3];
    wait_ms = word_buffer[4];

    if (word_count < 6)
    {
        verbose = 0;
    }
    else
    {
        verbose = word_buffer[5];
    }
    
    uart_write_string("Press "COMMAND_BREAK_STR" to stop:\r\n");

    switch_set(ant_start);
    
    ant_idx = ant_start;
    for (i=0; (!count) || (i<count); i++)
    {
        uart_write_hex_byte(ant_idx);
        uart_write(':');
        uart_write(' ');
        
        switch (rfid_read(poll_ms, ant_idx))
        {
            case RFID_READ_TIMEOUT:
                uart_write_string("FFFFFFFFFF\r\n");
                if (verbose)
                {
                    rfid_print_stats();
                    if (retry_parity > 0) rfid_print_raw_data();
                }
                break;
                
            case RFID_READ_OK:
                rfid_print_data();
                uart_write_nl();
                break;
        }

        if (uart_check_break()) break;

        // Do specified delay, switching antenna half-way in between
        
        if (wait_ms) timer_ms_delay(wait_ms >> 1);

        if (++ant_idx > ant_end) ant_idx = ant_start;

        switch_set(ant_idx);
        
        if (wait_ms) timer_ms_delay(wait_ms >> 1);
    }

    switch_set(SWITCH_ALL_OFF);

}
*/

/*
void slave_test()
{
    uint8_t ant_idx;
    
    uart_write_string("Slave: "COMMAND_BREAK_STR" to quit.\r\n");

    // First antenna, RFID off
    rfid_write_standby(0);
    ant_idx = system_config.ant_start;
    switch_set(ant_idx);

    for ( ; ; )
    {
        uart_write_hex_byte(ant_idx);
        uart_write(' ');
        
        // Wait for rising edge (or high level on first run)
        while (rfid_read_output() == 0)
        {
            if (uart_check_break()) goto slave_test_done;
        }
        uart_write('+');
        
        // Got a rising edge, so enable standby
        rfid_write_standby(1);
        
        // Wait for falling edge on OUTPUT pin
        while (rfid_read_output() != 0)
        {
            if (uart_check_break()) goto slave_test_done;
        }
        uart_write('-');
        uart_write_nl();
        
        // Got falling edge, disable standby and switch to next antenna
        rfid_write_standby(0);
        
        ant_idx++;
        if (ant_idx > system_config.ant_stop) ant_idx = system_config.ant_start;
        switch_set(ant_idx);
    }
        
 slave_test_done:
    rfid_write_standby(0);
    switch_set(SWITCH_ALL_OFF);
}
*/

void adc_test()
{
    uint16_t t1, t2, dt;
    uint16_t val, i;

    uart_write_string("Press "COMMAND_BREAK_STR" to quit.\r\n");
    
    for (i=0; i<10; i++)
    {
        adc_start();

        t1 = timer_ms_read_uint16();
        while (!adc_ready())
        {
            t2 = timer_ms_read_uint16();
            dt = t2-t1;
            if (dt > TEST_RUNTIME_MS)
            {
                adc_stop();
                uart_write_string("Timeout waiting for ADC.\r\n");
                return;
            }
        }
    
        val = adc_read();
        uart_write_dec_int(val);
        uart_write_nl();

        if (uart_check_break()) break;
    }
}

void battery_test()
{
    uint16_t v;

    uart_write_string("Main battery: ");
    v = battery_main_mv();
    uart_write_dec_int(v);
    uart_write_string(" mv\r\n");
    
    uart_write_string("Coin cell: ");
    v = battery_coin_mv();
    uart_write_dec_int(v);
    uart_write_string(" mv\r\n");
}

void time_test_timestamp_to_date()
{
    uint16_t t1, t2, dt;
    uint32_t ts_orig, ts;
    rtc_time_def t;
    uint32_t ops;

    //ts = 0x65721793;
    // Get a random but valid timestamp
    do
    {
        ts_orig = (uint32_t)rand() + ((uint32_t)rand() << 16);
    } while (ts_orig > 0xc5bfae7f);

    uart_write_string("Random timestamp: ");
    uart_write_dec_int(ts_orig);
    uart_write_string(" (0x");
    uart_write_hex_long(ts_orig);
    uart_write_string(")\r\n");
    
    uart_write_string("Timestamp to date...\r\n");
    
    t1 = timer_ms_read_uint16();

    ops = 0;
    for ( ; ; )
    {
        rtc_timestamp_to_date(ts_orig, &t);
        rtc_timestamp_to_date(ts_orig, &t);
        rtc_timestamp_to_date(ts_orig, &t);
        rtc_timestamp_to_date(ts_orig, &t);
        rtc_timestamp_to_date(ts_orig, &t);
        rtc_timestamp_to_date(ts_orig, &t);
        rtc_timestamp_to_date(ts_orig, &t);
        rtc_timestamp_to_date(ts_orig, &t);
        rtc_timestamp_to_date(ts_orig, &t);
        rtc_timestamp_to_date(ts_orig, &t);
        ops += 10;
        
        t2 = timer_ms_read_uint16();
        dt = (t2-t1);

        if (dt > TEST_RUNTIME_MS)
        {
            break;
        }
    }

    uart_write_string("Date: ");
    rtc_print_date_time(&t, 0);
    uart_write_nl();

    uart_write_string("Speed: ");
    uart_write_dec_int(ops);
    uart_write_string("/s\r\n");

    /* ==== */

    uart_write_string("Date to timestamp...\r\n");

    t1 = timer_ms_read_uint16();

    ops = 0;
    for ( ; ; )
    {
        ts = rtc_date_to_timestamp(&t);
        ts = rtc_date_to_timestamp(&t);
        ts = rtc_date_to_timestamp(&t);
        ts = rtc_date_to_timestamp(&t);
        ts = rtc_date_to_timestamp(&t);
        ts = rtc_date_to_timestamp(&t);
        ts = rtc_date_to_timestamp(&t);
        ts = rtc_date_to_timestamp(&t);
        ts = rtc_date_to_timestamp(&t);
        ts = rtc_date_to_timestamp(&t);
        ops += 10;
        
        t2 = timer_ms_read_uint16();
        dt = (t2-t1);

        if (dt > TEST_RUNTIME_MS)
        {
            break;
        }
    }

    uart_write_string("Timestamp: ");
    uart_write_dec_int(ts);
    uart_write_nl();

    uart_write_string("Speed: ");
    uart_write_dec_int(ops);
    uart_write_string("/s\r\n");

    if (ts != ts_orig)
    {
        uart_write_string("TIMESTAMP MISMATCH!\r\n");
    }
}

void time_test()
{
    switch(input_string_test[1])
    {

        case '1':
            time_test_timestamp_to_date();
            break;

        default:
            uart_write_string("d1  Test timing/accuracy of date<->timestamp.\r\n");
            break;
    }

}

void timer_test_ms()
{
    uint32_t t1, t2, dt;
    uint32_t sec, ms;

    uart_write_string("Testing ms timer (TMR2).  Press "COMMAND_BREAK_STR" to stop...\r\n");
    
    t1 = timer_ms_read();
    for ( ; ; )
    {
        t2 = timer_ms_read();
        dt = t2-t1;
        sec = dt / 1000;
        ms  = dt % 1000;
        uart_write_dec_int(sec);
        uart_write('.');
        uart_write_dec_int(ms);
        uart_write_nl();

        if (uart_check_break()) break;
    }

}

void timer_test()
{
    switch(input_string_test[1])
    {
        case '1':
            timer_test_ms();
            break;

        default:
            uart_write_string("t1  Test ms timer.\r\n");
            break;
    }
}

void sleep_test()
{
    uart_write_string("Going to sleep...\r\n");

    do_sleep();

    /*
    CPUDOZE &= ~(0x80);
    VB3GPR |= 0x1;

    GINT_OFF;
    
    timer_ms_sleep();
    rt_sleep();
    SLEEP();

    uart_init();
    uart_write_string("Woke up.  Resetting...\r\n");
    RESET();

    while(1);
    */
    
    //do_sleep();
}

void test_uart()
{
    for ( ; ; )
    {
        if (uart_read_ready())
        {
            uart_write_hex_byte(uart_read());
            uart_write_nl();
        }
    }
}

void log_test()
{
    uint16_t word_count;
    uint16_t i;
    uint8_t j;
    uint16_t t;
    
    if ((parse_get_numbers((parse_char *)(input_string_test+2),
                           word_buffer,
                           sizeof(word_buffer),
                           &word_count,
                           PARSE_INTEGER_SIZE_UINT32,
                           PARSE_RADIX_DEC)
         < 0) || (word_count < 1))
    {
        uart_write_string("l [num_tags]\n");
        return;
    }

    for (i=0; i<(word_buffer[0] & 0xffff); i++)
    {
        log_entry.rfid[4] = (i >> 8);
        log_entry.rfid[3] = (i & 0xff);
        log_entry.rfid[2] = (i & 0xf);
        log_entry.rfid[1] = (i & 0xff);
        log_entry.rfid[0] = (i >> 8);

        /*
        for (j=4; j != 0xff; j--)
        {
            //t = rt_timer_read();
            log_entry.rfid[j] = i1 & 0xff;
            i1 >>= 1;
        }
        */
        
        log_entry.time_stamp = rtc_date_to_timestamp(&rtc_time);
        log_entry.antenna_mask = 1 << (i & 0xf);
        tag_log_store_entry_eeprom();
    }
}

void print_test()
{
    uint16_t word_count;
    uint32_t i;
    uint8_t junk;
    
    if ((parse_get_numbers((parse_char *)(input_string_test+2),
                           word_buffer,
                           sizeof(word_buffer),
                           &word_count,
                           PARSE_INTEGER_SIZE_UINT32,
                           PARSE_RADIX_DEC)
         < 0) || (word_count < 1))
    {
        uart_write_string("p [num_chars]\n");
        return;
    }

    for (i=0; i<(word_buffer[0]); i++)
    {
        uart_write('A' + (i & 0xf));
        //eeprom_read_bytes(0x100, &junk, 1);
        if ((i & 0xf) == 0) 
        {
            uart_write_nl();
        }
    }
}
        
void test_print_help()
{
    uart_write_string("Commands:\r\n");
    uart_write_string("?  Help\r\n");
    uart_write_string("a  ADC test\r\n");
    uart_write_string("b  Battery test\r\n");
    uart_write_string("d  Test date/time functions\r\n");
    uart_write_string("e  EEPROM test\r\n");
    //uart_write_string("i  i2c test\r\n");
    uart_write_string("l  Put random tag IDs in log\r\n");
    //uart_write_string("r  RFID poll test\r\n");
    //uart_write_string("s  Slave test\r\n");
    uart_write_string("S  Sleep mode test\r\n");
    uart_write_string("t  Test timer functions\r\n");
    uart_write_string("q  Quit\r\n");
}

void test_panel()
{
    uint8_t quit, immediate_command_flag;
    uint8_t b;


    if (input_string[1] != 0)
    {
        // An immediate command follows the t command.
        immediate_command_flag = 1;
        input_string_test = input_string+1;
    }
    else
    {
        immediate_command_flag = 0;
        input_string_test = input_string;
    }

    quit = 0;
    while (!quit)
    {
        // Set timeout for autostart
        if (system_config.time_auto) uart_set_timeout(system_config.time_auto*1000);

        if (!immediate_command_flag)
        {
            uart_write_prompt("test> ");
            uart_read_string(input_string, sizeof(input_string), 1);
        }
        
        switch(input_string_test[0])
        {
            case 'a':
                adc_test();
                break;

            case 'b':
                battery_test();
                break;
                
            case 'd':
                time_test();
                break;

            case UART_TIMEOUT_CHAR:
                uart_write_string("Timeout.\r\n");
            case 'q':
                quit = 1;
                break;

            case 'e':
                eeprom_test();
                break;

            //case 'i':
            //    i2c_test();
            //    break;

            case 'l':
                log_test();
                break;
                
            case 'p':
                print_test();
                break;
                
//            case 'r':
//                rfid_test();
//                break;

//            case 's':
//                slave_test();
//                break;

            case 'S':
                sleep_test();
                break;

            case 't':
                timer_test();
                break;

//            case 'u':
//                test_uart();
//                break;
                
            case '?':
                test_print_help();
                break;

            default:
                command_error();
                break;
        }

        if (immediate_command_flag) break;
    }
}
