/*============================================================================
 * uart.h
 *      Header file for uart.c module.
 *==========================================================================*/

#ifndef _uart_h_
#define _uart_h_

#include <xc.h>
#include <inttypes.h>

#define UART_TIMEOUT_CHAR       255

#define UART_MODE_UNKNOWN   0x0
#define UART_MODE_BT        0x1
#define UART_MODE_RS232     0x2

#define UART_PROMPT_ECHO    (uart_mode == UART_MODE_RS232)

extern uint8_t uart_mode;

typedef char uart_char;

void uart_init(void);
void uart_write(uart_char ch);
void uart_write_string(uart_char *s);
void uart_write_dec_int(uint32_t n);
void uart_write_hex_byte(uint8_t v);
void uart_write_hex_long(uint32_t v);
void uart_read_string(uart_char *s, uint32_t buffer_size, uart_char echo);
void uart_write_prompt(uart_char *s);
uart_char uart_read(void);
uint8_t uart_write_ready(void);
uint8_t uart_read_ready(void);
void uart_write_nb(uart_char ch);
uart_char uart_read(void);
void uart_write_nl(void);
void uart_set_timeout(uint32_t timeout_val);
void uart_sleep(void);
uint8_t uart_check_break(void);
uint16_t uart_flush(uart_char *s, uint32_t buffer_size);
void uart_write_crc(void);
void uart_crc_on(void);
void uart_crc_off(void);
void uart_crc_reset(void);

#endif
